% CONDREGE.M
% Conditions for application of linear regression
% Guy Mlard, December 1996. gmelard@ulb.ac.be
% MATLAB with the Statistical Toolbox
% (DISTCHCK.M, NORMSCDF.M, RNDCHECK.M, TCDF.M, TRND.M)

% Traitement automatique ou choix
rep = menu('CONDREG, Guy Mlard, 1996. Treatment','automatic','non automatic') ;
if rep == 1 
  R = 1000 ;
  T = 10 ;
  H = 3 ;
  beta = 1 ;
else
  R = max(input('Enter the number of simulations '), 1);
  T = input('Enter the number of data ') ;
  H = input('Enter the forecasting horizon ') ;
  beta = input('Enter the regression coefficient ') ;
end
disp(['Number of simulations    : ', int2str(R)]) ;
disp(['Number of data           : ', int2str(T)]) ;
disp(['Forecasting horizon      : ', int2str(H)]) ;
disp(['Regression coefficient   : ', num2str(beta)]) ;
% Default values
if T <= 0, T = 10, end ;
if H <= 0, H = 1, end ;

while 1

C = menu('Enter the unsatisfied condition ', ...
    '(1) none', ...
    '(2) explanatory variable measured with errors', ...
    '(3) multicollinearity', ...
    '(4) non linear relation', ...
    '(5) non normality', ...
    '(6) outliers', ...
    '(7) heteroscedasticity', ...
    '(8) positive autocorrelation of errors', ...
    '(9) negative autocorrelation of errors', ...
    '(10) STOP' ) ;

if C == 10 ; break, end 

% I N I T I A  L I S A T I O N 

% Options 
k = 1              ; % normal law
pertu  = 2/T       ; % proportion of outliers
ampli  = 10        ; % amplitude of perturbations
ervar  = T^0.5     ; % variation of errors on the explanatory variable
quadr  = 20/(T^2)  ; % amount of non linearity
heter  = 2/T       ; % slope of the trend on the standard deviation
autoc  = min(0.9,2.5/(T^0.5)) ; % autocorrelation of errors
if C == 5      
  k = menu('Specify the law','normal','uniform','Laplace', ...
                           'Student with 2 d.f.', 'Cauchy') ;
end
% NEW The following instruction is not compatible with octave. 
% It toggles to a white background for plots. 
% Another call will give back black menus
% If the contrary occurs (white menus and black plots), then
% type the command "whitebg" once before calling "condrege"
whitebg;
X0 = 1:(T + H) ;
Z = zeros(1,T + H) ;
S = ones(1,T + H) ;
Q = zeros(1,T + H) ;
E = zeros(1,T + H) ;
compt1 = zeros(1,1001);
compt2 = zeros(1,1001);
compt3 = zeros(1,1001);

% T R E A T M E N T

% loop over replications
for ir = 1:R
  % we simulate the errors (and the explanatory variable)
  X = X0 ;
  if C == 2, Z = ervar*randn(1, T + H) ; end 
  if C == 3, X = 1 + X0/10000000000 ; end
  if C == 4, Q = quadr*(X0 - T/2).*(X0 - T/2); end 
  if     k == 1 , E = randn(1, T + H) ;
  elseif k == 2 , E = 2*sqrt(3)*(rand(1, T + H) - 0.5) ;
  elseif k == 3 , for i = 1 : (T + H)
                    temp1 = rand(1) ;
                    temp2 = abs( 2*temp1 - 1) ;
                    E(i) =  (1/sqrt(2))*log(1 - temp2) ;
                    if temp1 < 0.5 ; E(i) = - E(i) ; end 
                  end 
  elseif k == 4 , E = trnd(2, 1, T + H) ;
  elseif k == 5 , E = trnd(1, 1, T + H) ;
  end
  if C == 6
    ALEA = rand(1, T + H) ;
    for i = 1 : (T + H)
      if ALEA(i) < pertu , E(i) = E(i) + ampli ; end
    end 
  end 
  if C == 7 , E = E.*(0.5 + heter*X0) ; end ;
  if C == 8 , for i = 2 : (T + H)
                 E(i) =  E(i) + autoc*E(i - 1) ;
              end 
  end
  if C == 9 , for i = 2 : (T + H)
                 E(i) =  E(i) - autoc*E(i - 1) ;
              end 
  end
  % we compute the theoretical relation
  XTH = X + Z ;
  YTH = 1 + beta*XTH + Q ;
  % we compute the observations of the dependent variable
  Y = YTH + E ;
  % we prepare the plot of distribution of the errors (at time T + h)
  i = min(1000, max(0, round((E(T + H) + 5)*100))) + 1 ;
  compt1(i) = compt1(i) + 1 ;
  % we perform linear simple linear regression
  covar = cov([Y(1:T)' X(1:T)']) ;
  xbar = mean(X(1:T)) ;
  ybar = mean(Y(1:T)) ;
  s2x = covar(2,2) ;
  s2y = covar(1,1) ;
  sxy = covar(1,2) ;
  b = sxy/s2x ;
  a = ybar - b*xbar ;
  s2b = (s2y/s2x - b^2)/(T - 2) ;
  s2e = s2y*(1 - (sxy^2)/(s2x*s2y))*(T - 1)/(T - 2) ;
  tstat = (b - beta)/sqrt(s2b) ;
  % we prepare the plot of the Student statistics
  i = min(1000, max(0, round((tstat + 5)*100))) + 1 ;
  compt2(i) = compt2(i) + 1 ;
  % we compute the residuals and the Studentised residuals
  YAJ = a + b*X ;
  RES = Y - YAJ ;
  VRES = s2e*(1 + 1/T + ((X(T + H) - xbar)^2)/((T - 1)*s2x)) ;
  NRES = RES(T + H)/(VRES^0.5) ;
  % we prepare the plot of errors (at time T + h)
  i = min(1000, max(0, round((NRES + 5)*100))) + 1 ;
  compt3(i) = compt3(i) + 1 ;
  % we perform the three plots for the first simulation
  if ir ==1
    if C == 1
      Titre = 'All the conditions are satisfied ' ;
    elseif C == 2
      Titre = ['Errors on the explanatory variable with standard deviation ', ...
               num2str(ervar) ] ;
    elseif C == 3
      Titre = 'Multicollinearity ' ;
    elseif C == 4
      Titre = ['Quadratic relation with coefficient ', ...
               num2str(quadr) ] ;
    elseif C == 5
      Titre = ['Law of errors number ', ...
               int2str(k) ] ;
    elseif C == 6
      Titre = ['Outliers with amplitude ', num2str(ampli), ...
               ', and probability ', num2str(pertu) ] ;
    elseif C == 7
      Titre = ['Heteroscedasticity with standard deviation from 0,5 to ', ...
               num2str(heter*T) ] ;
    elseif C == 8
      Titre = ['Positive autocorrelation of errors equal to ', ...
               num2str(autoc) ] ;
    elseif C == 9
      Titre = ['Negative autocorrelation of errors equal to -', ...
               num2str(autoc) ] ;
    end
    % we perform a plot with the data, the theoretical regression,
    % and the fitted regression
    disp(Titre) ;
    plot(X(1:T), Y(1:T), 'ro')
    hold on
    plot(X(1:T + H), YAJ(1:T + H), 'r--')
    hold on
    plot(XTH(1:T + H), YTH(1:T + H), 'g-')
    hold off
    title([Titre, '. Data of an example'])
    xlabel('X')
    ylabel('Y obs.(o), theor.(-), adj.(- -)')
    pause
    % we perform the plot of residuals in function of adjusted values
    LIMP = +1.96*ones(1, T) ;
    LIMM = -1.96*ones(1, T) ;
    plot(YAJ(1:T), RES(1:T), 'rx')
    hold on
    plot(YAJ(1:T), LIMP, 'g:') ;
    hold on
    plot(YAJ(1:T), LIMM, 'g:') ;    
    hold off
    title([Titre, '. Analysis of residuals'])
    xlabel('adjusted value')
    ylabel('residual')
    pause
    % we perform the plot of residuals in function of time
    plot(X0(1:T), RES(1:T), 'r-')
    hold on
    plot(X0(1:T), LIMP, 'g:') ;
    hold on
    plot(X0(1:T), LIMM, 'g:') ;    
    hold off
    title([Titre, '. Analysis of residuals'])
    xlabel('time')
    ylabel('residual')
    pause
  end
end

% C L O S I N G

for i = 2:1001
  compt1(i) = compt1(i - 1) + compt1(i) ;
  compt2(i) = compt2(i - 1) + compt2(i) ;
  compt3(i) = compt3(i - 1) + compt3(i) ;
end
nv = ((1:1001) - 501)/100 ;
fdn = normscdf(nv) ;
compt1 = compt1/R ; 
plot(nv, compt1, 'r:')
hold on
plot(nv, fdn,'g-')
hold off
title([Titre, '. Distribution of errors'])
xlabel(['Error at time ', int2str(T + H)])
ylabel('probability(-), frequency(- -)')
pause
tv = ((1:1001) - 501)/100 ;
fdt = tcdf(tv, T - 2) ;
compt2 = compt2/R ; 
plot(tv, compt2, 'r:')
hold on
plot(tv, fdt,'g-')
hold off
title([Titre, '. Student t statistic'])
xlabel('t statistic')
ylabel('probability(-), frequency(- -)')
pause
nv = ((1:1001) - 501)/100 ;
fdn = normscdf(nv) ;
compt3 = compt3/R ; 
plot(nv, compt3, 'r:')
hold on
plot(nv, fdn,'g-')
hold off
title([Titre, '. Distribution of residuals'])
xlabel(['Residual at time ', int2str(T + H)])
ylabel('probability(-), frequency(- -)')
pause
% NEW
whitebg;

end